# Generated by the protocol buffer compiler.  DO NOT EDIT!
# sources: cosmos/genutil/v1beta1/genesis.proto
# plugin: python-betterproto
from dataclasses import dataclass
from typing import List

import betterproto


@dataclass(eq=False, repr=False)
class GenesisState(betterproto.Message):
    """GenesisState defines the raw genesis transaction in JSON."""

    gen_txs: List[bytes] = betterproto.bytes_field(1)
    """gen_txs defines the genesis transactions."""
